/* Implementation of `apply_3dcc' and `save_3dcc' */

#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define WIN32_LEAN_AND_MEAN
#include <windows.h>

#include "3DCC_p.h"




#define COLOR_3DALTFACE 25  /* missing index */

#define CS_3D_DARK_SHADOW		COLOR_3DDKSHADOW
#define CS_3D_SHADOW			COLOR_3DSHADOW
#define CS_3D_FACE			COLOR_3DFACE
#define CS_3D_ALTERNATE_FACE		COLOR_3DALTFACE
#define CS_3D_LIGHT			COLOR_3DLIGHT
#define CS_3D_HILIGHT			COLOR_3DHILIGHT
#define CS_ACTIVE_CAPTION		COLOR_ACTIVECAPTION
#define CS_ACTIVE_CAPTION_GRADIENT	COLOR_GRADIENTACTIVECAPTION
#define CS_ACTIVE_CAPTION_TEXT		COLOR_CAPTIONTEXT
#define CS_ACTIVE_WINDOW_BORDER		COLOR_ACTIVEBORDER
#define CS_APPLICATION_BACKGROUND	COLOR_APPWORKSPACE
#define CS_BUTTON_TEXT			COLOR_BTNTEXT
#define CS_DESKTOP			COLOR_DESKTOP
#define CS_FOCUSED_OBJECT_FRAME		COLOR_WINDOWFRAME
#define CS_GRAY_TEXT			COLOR_GRAYTEXT
#define CS_INACTIVE_CAPTION		COLOR_INACTIVECAPTION
#define CS_INACTIVE_CAPTION_GRADIENT	COLOR_GRADIENTINACTIVECAPTION
#define CS_INACTIVE_CAPTION_TEXT	COLOR_INACTIVECAPTIONTEXT
#define CS_INACTIVE_CAPTION_BORDER	COLOR_INACTIVEBORDER
#define CS_MENU				COLOR_MENU
#define CS_MENU_TEXT			COLOR_MENUTEXT
#define CS_MOUSE_HIGHLIGHT		COLOR_HOTLIGHT
#define CS_SCROLLBAR			COLOR_SCROLLBAR
#define CS_SELECTED_ITEM		COLOR_HIGHLIGHT
#define CS_SELECTED_ITEM_TEXT		COLOR_HIGHLIGHTTEXT
#define CS_TOOLTIP			COLOR_INFOBK
#define CS_TOOLTIP_TEXT			COLOR_INFOTEXT
#define CS_WINDOW			COLOR_WINDOW
#define CS_WINDOW_TEXT			COLOR_WINDOWTEXT

static int CS_KEYS[] = {
	CS_3D_DARK_SHADOW,
	CS_3D_SHADOW,
	CS_3D_FACE,
	CS_3D_ALTERNATE_FACE,
	CS_3D_LIGHT,
	CS_3D_HILIGHT,
	CS_ACTIVE_CAPTION,
	CS_ACTIVE_CAPTION_GRADIENT,
	CS_ACTIVE_CAPTION_TEXT,
	CS_ACTIVE_WINDOW_BORDER,
	CS_APPLICATION_BACKGROUND,
	CS_BUTTON_TEXT,
	CS_DESKTOP,
	CS_FOCUSED_OBJECT_FRAME,
	CS_GRAY_TEXT,
	CS_INACTIVE_CAPTION,
	CS_INACTIVE_CAPTION_GRADIENT,
	CS_INACTIVE_CAPTION_TEXT,
	CS_INACTIVE_CAPTION_BORDER,
	CS_MENU,
	CS_MENU_TEXT,
	CS_MOUSE_HIGHLIGHT,
	CS_SCROLLBAR,
	CS_SELECTED_ITEM,
	CS_SELECTED_ITEM_TEXT,
	CS_TOOLTIP,
	CS_TOOLTIP_TEXT,
	CS_WINDOW,
	CS_WINDOW_TEXT
};




/* ---------------- apply_3dcc ---------------- */

static const char*
_BBCmdApply3DCCCommand_initialize( BBCmdCommand* _self,
                                   int ac, const char* const* av,
                                   const BBCmdCompiler* compiler )
{
	BBCmdApply3DCCCommand* self;

	assert(_self->cmd_descriptor == BBCmdApply3DCCCommandDescriptor);
	self = (BBCmdApply3DCCCommand*)_self;

	self->a3_scheme = NULL;

	if (ac - 1 != 1) return "Incorrect number of arguments";

	self->a3_scheme = strdup(av[1]);
	if (self->a3_scheme == NULL)
		return "Out of memory(_BBCmdApply3DCCCommand_initialize)";

	return NULL;
}


static void
_BBCmdApply3DCCCommand_del(BBCmdCommand* _self)
{
	free(((BBCmdApply3DCCCommand*)_self)->a3_scheme);
}


static int
_BBCmdApply3DCCCommand_execute(const BBCmdCommand* _self)
{
	BBCmdApply3DCCCommand* self;
	FILE* file;
	COLORREF scheme[BBCmd_arraysizeof(CS_KEYS)];
	int i;
	char line[256];

	assert(_self->cmd_descriptor == BBCmdApply3DCCCommandDescriptor);
	self = (BBCmdApply3DCCCommand*)_self;

	file = fopen(self->a3_scheme, "rt");
	if (file == NULL) return BBCmd_RC_FALSE;

	for (i = 0; i < BBCmd_arraysizeof(CS_KEYS); i++) {
		if (fgets(line, BBCmd_arraysizeof(line), file) == NULL)
			break;

		scheme[i] = (COLORREF)atol(line);
	}

	for (; i < BBCmd_arraysizeof(CS_KEYS); i++) {
		scheme[i] = RGB(0, 0, 0);
	}

	fclose(file);

	SetSysColors(BBCmd_arraysizeof(CS_KEYS), CS_KEYS, scheme);

	return BBCmd_RC_TRUE;
}




const BBCmdCommandDescriptor _BBCmdApply3DCCCommandDescriptor = {
	"apply_3dcc",
	NULL,
	"(scheme)\n"
		"Apply specified color scheme.\n"
		"`scheme' must be a color scheme of 3D Color Changer.",
	sizeof(BBCmdApply3DCCCommand),
	_BBCmdApply3DCCCommand_initialize,
	_BBCmdApply3DCCCommand_del,
	_BBCmdApply3DCCCommand_execute
};




/* ---------------- save_3dcc ---------------- */

static const char*
_BBCmdSave3DCCCommand_initialize( BBCmdCommand* _self,
                                  int ac, const char* const* av,
                                  const BBCmdCompiler* compiler )
{
	BBCmdSave3DCCCommand* self;

	assert(_self->cmd_descriptor == BBCmdSave3DCCCommandDescriptor);
	self = (BBCmdSave3DCCCommand*)_self;

	self->s3_scheme = NULL;

	if (ac - 1 != 1) return "Incorrect number of arguments";

	self->s3_scheme = strdup(av[1]);
	if (self->s3_scheme == NULL)
		return "Out of memory(_BBCmdSave3DCCCommand_initialize)";

	return NULL;
}


static void
_BBCmdSave3DCCCommand_del(BBCmdCommand* _self)
{
	free(((BBCmdSave3DCCCommand*)_self)->s3_scheme);
}


static int
_BBCmdSave3DCCCommand_execute(const BBCmdCommand* _self)
{
	BBCmdSave3DCCCommand* self;
	FILE* file;
	int i;

	assert(_self->cmd_descriptor == BBCmdSave3DCCCommandDescriptor);
	self = (BBCmdSave3DCCCommand*)_self;

	file = fopen(self->s3_scheme, "wt");
	if (file == NULL) return BBCmd_RC_FALSE;

	for (i = 0; i < BBCmd_arraysizeof(CS_KEYS); i++)
		fprintf(file, "%ld\n", GetSysColor(CS_KEYS[i]));

	fclose(file);

	return BBCmd_RC_TRUE;
}




const BBCmdCommandDescriptor _BBCmdSave3DCCCommandDescriptor = {
	"save_3dcc",
	NULL,
	"(scheme)\n"
		"Save the current color scheme to `scheme'\n"
		"as a color scheme of 3D Color Changer.",
	sizeof(BBCmdSave3DCCCommand),
	_BBCmdSave3DCCCommand_initialize,
	_BBCmdSave3DCCCommand_del,
	_BBCmdSave3DCCCommand_execute
};




/* __END__ */
