/**
 * Implementation of the framework of BBCmd
 *
 * FIXME: Too slow anything.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "BBCmd_p.h"

#include "Constant.h"
#include "Multi.h"
#include "Loop.h"
#include "Random.h"

#include "3DCC.h"
#include "Exec.h"
#include "File.h"
#include "Focus.h"
#include "Message.h"
#include "MiscCmds.h"
#include "Mouse.h"
#include "Pause.h"
#include "Sound.h"
#include "Thread.h"
#include "Test.h"
#include "Window.h"
#include "Workspace.h"








BBCmdAPI_FUNC(BBCmdCompiler*)
BBCmdCompiler_New(void)
{
	const BBCmdCommandDescriptor* default_cd_table[] = {
		BBCmdTestCommandDescriptor,
		BBCmdPauseCommandDescriptor,
		BBCmdSoundCommandDescriptor,
		BBCmdExecCommandDescriptor,
		BBCmdFileCommandDescriptor,
		BBCmdFocusCommandDescriptor,
		BBCmdWorkspaceCommandDescriptor,
		BBCmdMouseMoveCommandDescriptor,
		BBCmdMouseWheelCommandDescriptor,
		BBCmdMouseClickCommandDescriptor,
		BBCmdVersionCommandDescriptor,
		BBCmdWindowMoveCommandDescriptor,
		BBCmdWindowResizeCommandDescriptor,
		BBCmdSetWallpaperCommandDescriptor,
		BBCmdApply3DCCCommandDescriptor,
		BBCmdSave3DCCCommandDescriptor,

		BBCmd_MC_DESCRIPTOR(aboutplugins),
		BBCmd_MC_DESCRIPTOR(aboutstyle),
		BBCmd_MC_DESCRIPTOR(adddesk),
		BBCmd_MC_DESCRIPTOR(broam),
		BBCmd_MC_DESCRIPTOR(deldesk),
		BBCmd_MC_DESCRIPTOR(deskleft),
		BBCmd_MC_DESCRIPTOR(deskright),
		BBCmd_MC_DESCRIPTOR(editblackbox),
		BBCmd_MC_DESCRIPTOR(editextensions),
		BBCmd_MC_DESCRIPTOR(editmenu),
		BBCmd_MC_DESCRIPTOR(editplugins),
		BBCmd_MC_DESCRIPTOR(editstyle),
		BBCmd_MC_DESCRIPTOR(editworkspacename),
		BBCmd_MC_DESCRIPTOR(gatherwindows),
		BBCmd_MC_DESCRIPTOR(hibernate),
		BBCmd_MC_DESCRIPTOR(hidemenu),
		BBCmd_MC_DESCRIPTOR(label),
		BBCmd_MC_DESCRIPTOR(lockworkstation),
		BBCmd_MC_DESCRIPTOR(logoff),
		BBCmd_MC_DESCRIPTOR(mainmenu),
		BBCmd_MC_DESCRIPTOR(quit),
		BBCmd_MC_DESCRIPTOR(reboot),
		BBCmd_MC_DESCRIPTOR(reconfig),
		BBCmd_MC_DESCRIPTOR(restart),
		BBCmd_MC_DESCRIPTOR(run),
		BBCmd_MC_DESCRIPTOR(shutdown),
		BBCmd_MC_DESCRIPTOR(slitmenu),
		BBCmd_MC_DESCRIPTOR(style),
		BBCmd_MC_DESCRIPTOR(stylesmenu),
		BBCmd_MC_DESCRIPTOR(suspend),
		BBCmd_MC_DESCRIPTOR(systembarmenu),
		BBCmd_MC_DESCRIPTOR(toggleplugins),
		BBCmd_MC_DESCRIPTOR(toggleslit),
		BBCmd_MC_DESCRIPTOR(togglesystembar),
		BBCmd_MC_DESCRIPTOR(toggletoolbar),
		BBCmd_MC_DESCRIPTOR(toggletray),
		BBCmd_MC_DESCRIPTOR(toolbarmenu),
		BBCmd_MC_DESCRIPTOR(workspacemenu),
		NULL
	};
	const BBCmdCommandDescriptor** d;
	BBCmdCompiler* self;
	int r;

	self = BBCmdCompiler_NewVanilla();
	if (self == NULL) return NULL;

	for (d = default_cd_table; *d != NULL; d++) {
		r = BBCmdCompiler_AddNewCommand(self, *d);
		if (!r) {BBCmdCompiler_Del(self); return NULL;}
	}

	return self;
}


BBCmdAPI_FUNC(BBCmdCompiler*)
BBCmdCompiler_NewVanilla(void)
{
	const BBCmdCommandDescriptor* default_cd_table[] = {
		BBCmd_CO_DESCRIPTOR(break),
		BBCmd_CO_DESCRIPTOR(continue),
		BBCmd_CO_DESCRIPTOR(false),
		BBCmd_CO_DESCRIPTOR(true),
		NULL
	};
	const BBCmdCommandDescriptor** d;
	BBCmdCompiler* self;
	int r;

	self = (BBCmdCompiler*)malloc(sizeof(*self));
	if (self == NULL) return NULL;

	self->cd_table = NULL;
	self->cd_table_size = 0;

	for (d = default_cd_table; *d != NULL; d++) {
		r = BBCmdCompiler_AddNewCommand(self, *d);
		if (!r) {BBCmdCompiler_Del(self); return NULL;}
	}

	return self;
}


BBCmdAPI_FUNC(void)
BBCmdCompiler_Del(BBCmdCompiler* self)
{
	free(self->cd_table);
	free(self);
}


BBCmdAPI_FUNC(int)
BBCmdCompiler_AddNewCommand( BBCmdCompiler* self,
                             const BBCmdCommandDescriptor* new_command )
{
	const BBCmdCommandDescriptor** new_table;

	new_table = (const BBCmdCommandDescriptor**)realloc(
		self->cd_table,
		sizeof(self->cd_table[0]) * (self->cd_table_size+1)
	);
	if (new_table == NULL) return 0;

	self->cd_table = new_table;
	self->cd_table[self->cd_table_size] = new_command;
	self->cd_table_size++;

	return !0;
}


BBCmdAPI_FUNC(const BBCmdCommandDescriptor* const*)
BBCmdCompiler_AvailableCommand(const BBCmdCompiler* self, int* n)
{
	if (n != NULL) *n = self->cd_table_size;
	return self->cd_table;
}


static const BBCmdCommandDescriptor*
_BBCmdCompiler_FindDescriptor( const BBCmdCompiler* self,
                               const char* command_name )
{
	int i;
	int j;

	for (i = 0; i < self->cd_table_size; i++) {
		if (!strcmp(command_name, self->cd_table[i]->cmd_name))
			return self->cd_table[i];
		if (self->cd_table[i]->cmd_alias != NULL) {
			j = 0;
			while (self->cd_table[i]->cmd_alias[j] != NULL) {
				if (!strcmp( command_name,
				             self->cd_table[i]->cmd_alias[j] ))
					return self->cd_table[i];
				j++;
			}
		}
	}

	return NULL;
}








BBCmdAPI_FUNC(void)
BBCmdCommand_Del(BBCmdCommand* self)
{
	if (self == NULL) return;

	(*(self->cmd_descriptor->cmd_del))(self);
	free(self);
}

BBCmdAPI_FUNC(int)
BBCmdCommand_Execute(const BBCmdCommand* self)
{
	int rc;

	rc = (*(self->cmd_descriptor->cmd_execute))(self);

	return (rc < 0) ? BBCmd_RC_TRUE : rc;
}

BBCmdAPI_FUNC(int)
_BBCmdCommand_Execute(const BBCmdCommand* self)
{
	return (*(self->cmd_descriptor->cmd_execute))(self);
}








BBCmdAPI_FUNC(const char*)
BBCmdCommandDescriptor_NameOf(const BBCmdCommandDescriptor* self)
	{return self->cmd_name;}
BBCmdAPI_FUNC(const char* const*)
BBCmdCommandDescriptor_AliasOf(const BBCmdCommandDescriptor* self)
	{return self->cmd_alias;}
BBCmdAPI_FUNC(const char*)
BBCmdCommandDescriptor_DocOf(const BBCmdCommandDescriptor* self)
	{return self->cmd_doc;}








BBCmdAPI_FUNC(HWND)
BBCmd_FindCore(void)
{
#if 0 /* don't cache -- this will not be a bottleneck. */
	const char* names[] = {    /* class name, window name */
		"BlackboxClass", "Blackbox",
		"xoblite", NULL
	};
	int i;
	static HWND core = NULL;

	/* BUGS: Assuming the core won't terminate
	 *       while running the application using BBCmd. */
	if (core != NULL && IsWindow(core)) return core;

	for (i = 0; i < BBCmd_arraysizeof(names); i += 2) {
		core = FindWindow(names[i+0], names[i+1]);
		if (core != NULL && IsWindow(core))
			return core;
	}

	return core = NULL;
#else
	const char* names[] = {    /* class name, window name */
		"BlackboxClass", "Blackbox",
		"xoblite", NULL
	};
	int i;
	HWND core;

	for (i = 0; i < BBCmd_arraysizeof(names); i += 2) {
		core = FindWindow(names[i+0], names[i+1]);
		if (core != NULL && IsWindow(core))
			return core;
	}

	return NULL;
#endif
}


BBCmdAPI_FUNC(HWND)
BBCmd_FindProxy(void)
{
#if 0 /* don't cache -- this will not be a bottleneck. */
	static HWND proxy = NULL;
	
	/* BUGS: Assuming the proxy won't terminate
	 *       while running the application using BBCmd. */
	if (proxy != NULL && IsWindow(proxy)) return proxy;

	proxy = FindWindow("BBKontroller-Proxy/Class", "BBKontroller-Proxy");

	return IsWindow(proxy) ? proxy : (proxy = NULL);
#else
	HWND proxy;
	
	proxy = FindWindow("BBKontroller-Proxy/Class", "BBKontroller-Proxy");

	return IsWindow(proxy) ? proxy : (proxy = NULL);
#endif
}


BBCmdAPI_FUNC(int)
BBCmd_AmIPlugin(void)
{
#if 0 /* don't cache -- this will not be a bottleneck. */
	static int result;
	static HWND core = NULL;
	DWORD my_pid;
	DWORD core_pid;

	/* BUGS: Assuming the core won't terminate
	 *       while running the application using BBCmd. */
	if (core != NULL && IsWindow(core)) return result;

	core = find_core();
	GetWindowThreadProcessId(core, &core_pid);
	my_pid = GetCurrentProcessId();

	result = (my_pid == core_pid);

	return result;
#else
	HWND core;
	DWORD my_pid;
	DWORD core_pid;

	core = BBCmd_FindCore();
	GetWindowThreadProcessId(core, &core_pid);
	my_pid = GetCurrentProcessId();

	return my_pid == core_pid;

#endif
}


typedef struct {
	int version;
	int cause;
} _version_cause_tuple;

#define _A BBCmd_RELEASE_LEVEL_ALPHA
#define _F BBCmd_RELEASE_LEVEL_FINAL
static const _version_cause_tuple _VERSION_CAUSE_TABLE[] = {
	{_BBCmd_VERSION(0, 1, 0, _F), BBCmd_IC_INCOMPATIBLE_API},
	{_BBCmd_VERSION(0, 0, 4, _A), BBCmd_IC_INCOMPATIBLE_INTERNAL},
	{_BBCmd_VERSION(0, 0, 3, _A), BBCmd_IC_INCOMPATIBLE_INTERNAL},
	{_BBCmd_VERSION(0, 0, 2, _A), BBCmd_IC_INCOMPATIBLE_INTERNAL},
	{-1, -1},
};
#undef _F
#undef _A

BBCmdAPI_FUNC(int)
BBCmd_IsCompatible(int version, int hint)
{
	const _version_cause_tuple* p;

	if (BBCmd_VERSION < version) return BBCmd_IC_TOO_OLD_DLL;

	for (p = _VERSION_CAUSE_TABLE; p->cause != -1; p++) {
		if (BBCmd_VERSION < p->version) continue;

		if ( version < p->version
		     && !( !(hint & BBCmd_ICH_KNOW_INTERNAL)
		           && (p->cause == BBCmd_IC_INCOMPATIBLE_INTERNAL) ) )
			return p->cause;

	}

	return BBCmd_IC_COMPATIBLE;
}








/**
 * Compiling-related code
 * ======================
 *
 * Compiling phases are:
 *
 * 1. Lexical analysis - convert token sequence from string.
 * 2. Parse token sequence and make commands.
 *
 *
 * Productions
 * -----------
 *
 * script: command*
 * command: simple_command | multi_command | loop_command | ...
 *
 * simple_command: NAME (LPAR (arg (COMMA arg)* COMMA?)?  RPAR)?
 * multi_command: LBRA command* RBRA
 * loop_command: LOOP NUMBER COMMAND
 * random_command: RANDOM multi_command
 * thread_command: THREAD command
 *
 * arg: STRING | NUMBER
 *
 *
 * Any WS and COMMENT may appear between a token and another.
 * They are removed lexical analysis phase.
 *
 *
 * Tokens
 * ------
 *
 * LOOP      'loop'
 * RANDOM    'random'
 * THREAD    'thread'
 *
 * NAME      ![A-Za-z_][A-Za-z_0-9]*!
 * STRING        /"([^\\"]|\\[\\"])*"/    # backslash escape
 *           or  /'[^']*'/
 * NUMBER    ![1-9][0-9]*! or !0[oO]?[0-7]+!
 *           or !0[xX][0-9A-Fa-f]+! or !0[bB][01]+!
 *
 * COMMA     ','
 * LPAR      '('
 * RPAR      ')'
 * LBRA      '{'
 * RBRA      '}'
 *
 * WS        /[ \t\n]+/
 * COMMENT   !#.*!
 */


typedef enum {
	TI_END,    /* internal use.  not a type of token */

	TI_LOOP,
	TI_RANDOM,
	TI_THREAD,

	TI_NAME,
	TI_STRING,
	TI_NUMBER,

	TI_COMMA,
	TI_LPAR,
	TI_RPAR,
	TI_LBRA,
	TI_RBRA,

	TI_WS,
	TI_COMMENT
} token_id;


typedef struct {
	token_id id;
	char* attr;
	int line;
	int column;
} token;


typedef struct {    /* TOken seQUENCE */
	char* _mem;
	int num;
	token* tokens;
	int current;
} toquence;


typedef struct {
	int line;
	int column;
	const char* message;
} toquence_error;




static int isfirstletterofname(int c)
	{return ('a'<=c && c<='z') || ('A'<=c && c<='Z') || (c=='_');}
static int issecondletterofname(int c)
	{return isfirstletterofname(c) || ('0' <= c && c <= '9');}
static int isbinarydigit(int c) {return '0' == c || c == '1';}
static int isdecimaldigit(int c) {return '0' <= c && c <= '9';}
static int isoctaldigit(int c) {return '0' <= c && c <= '7';}
static int ishexaldigit(int c)
{
	return    ('0' <= c && c <= '9')
	       || ('a' <= c && c <= 'f') || ('A' <= c && c <= 'F');
}


typedef struct {
	token_id id;
	const char* s;
} _token_id_string_tuple;

static const _token_id_string_tuple _TOKEN_ID_STRING_TABLE[] = {
	{TI_LOOP, "loop"},
	{TI_RANDOM, "random"},
	{TI_THREAD, "thread"},
	{TI_END, NULL}
};

static token_id
id_of(const char* unterminated_string, int len)
{
	const _token_id_string_tuple* p;

	for (p = _TOKEN_ID_STRING_TABLE; p->id != TI_END; p++)
		if ( len == strlen(p->s)
		     && !strncmp(unterminated_string, p->s, len) )
			return p->id;

	return TI_NAME;
}


static int
number_attr_as_int(const char* attr)
{
	if (attr[0] == '0') {
		int base;
		switch (attr[1]) {
		default:
		case 'b': case 'B': base =  2; break;
		case 'o': case 'O': base =  8; break;
		case 'x': case 'X': base = 16; break;
		}
		return (int)strtol(attr+2, NULL, base);
	} else {
		return atoi(attr);
	}
}


/**
 * Return a sequence of tokens from a given `string'.
 * If this failed, then return NULL
 * and store the information in `*error_info'.
 */
static toquence*
toquence_from_string(const char* string, toquence_error* error_info)
{
	toquence* self;

	char* p;
	char* attr;
	token_id id;
	token* tmp;
	int lineno;
	char* first_column;
	char* token_column;

	self = (toquence*)malloc(sizeof(*self));
		if (self == NULL) {
			error_info->line = 0;
			error_info->column = 0;
			error_info->message = "Out of memory (toquence)";
			goto e_oom_toquence;
		}
	self->current = 0;
	self->num = 0;
	self->tokens = NULL;
	self->_mem = strdup(string);
		if (self->_mem == NULL) {
			error_info->line = 0;
			error_info->column = 0;
			error_info->message ="Out of memory (toquence->_mem)";
			goto e_oom_strdup;
		}

	lineno = 1;  first_column = token_column = self->_mem;
	p = self->_mem;
	while (*p != '\0') {
		switch (*p) {
		/* WS */
		case '\n':
			lineno++;  first_column = p+1;
		case ' ':
		case '\t':
			*p = '\0';  p++;
			continue;
		/* COMMENT */
		case '#':
			*p = '\0';  p++;
			while (*p != '\n' && *p != '\0')
				p++;
			if (*p == '\n')
				{lineno++;  first_column = p+1;}
			continue;
		/* Others */
		case ',': id = TI_COMMA;    attr = NULL;
		          token_column = p;  *p = '\0'; p++;  break;
		case '(': id = TI_LPAR;     attr = NULL;
		          token_column = p;  *p = '\0'; p++;  break;
		case ')': id = TI_RPAR;     attr = NULL;
		          token_column = p;  *p = '\0'; p++;  break;
		case '{': id = TI_LBRA;     attr = NULL;
		          token_column = p;  *p = '\0'; p++;  break;
		case '}': id = TI_RBRA;     attr = NULL;
		          token_column = p;  *p = '\0'; p++;  break;
		case '\'':
			token_column = p;
			*p = '\0';  p++;
			attr = p;

			while (!0) {
				if (*p == '\'' || *p == '\0') break;
				if (*p == '\n') {lineno++; first_column = p+1;}
				p++;
			}
			if (*p != '\'') {
				error_info->message = "Unterminated string";
				goto e_unterminated_string;
			}
			*p = '\0';
			p++;
			id = TI_STRING;
			break;
		case '"': {
			char* q;

			token_column = p;
			*p = '\0';  p++;
			attr = q = p;

			while (!0) {
				if (*p == '"' || *p == '\0') break;
				if (*p == '\n') {lineno++; first_column = p+1;}

				if (*p == '\\') {
					p++;
					if (*p != '\\' && *p != '"') {
					  error_info->message
					    = "Invalid char in string";
					  goto e_invalid_char_in_string;
					}
				}

				*q = *p;  p++;  q++;
			}
			if (*p != '"') {
				error_info->message = "Unterminated string";
				goto e_unterminated_string;
			}
			p++;
			*q = '\0';
			id = TI_STRING;
			} break;
		default:
			token_column = p;
			if (isfirstletterofname(*p)) {
				attr = p;

				do {p++;} while (issecondletterofname(*p));

				id = id_of(attr, (p - attr) / sizeof(char));
			} else if (isdecimaldigit(*p)) {
				int (*isdigit)(int);
				attr = p;
				if ('0' == *p) {
					p++;
					if (*p == 'x' || *p == 'X')
						{p++; isdigit = ishexaldigit;}
					else if (*p == 'o' || *p == 'O')
						{p++; isdigit = isoctaldigit;}
					else if (*p == 'b' || *p == 'B')
						{p++; isdigit = isbinarydigit;}
					else
						{isdigit = isoctaldigit;}
				} else
					isdigit = isdecimaldigit;
				while ((*isdigit)(*p))
					p++;
				if ( isdecimaldigit(*p)
				     || isfirstletterofname(*p) ) {
					error_info->message
						= "Invalid char in number";
					goto e_invalid_char_in_number;
				}
				id = TI_NUMBER;
			} else {
				error_info->message = "Invalid char";
				goto e_invalid_char;
			}
			break;
		}

		tmp = (token*)realloc( self->tokens,
		                        sizeof(token) * (self->num+1) );
		if (tmp == NULL) goto e_oom_realloc;
		self->tokens = tmp;
		self->tokens[self->num].id = id;
		self->tokens[self->num].attr = attr;
		self->tokens[self->num].column = (token_column - first_column)
		                                      / sizeof(char) + 1;
		self->tokens[self->num].line = lineno;
		self->num++;
	}

	return self;

e_oom_realloc:
e_invalid_char_in_string:
e_unterminated_string:
e_invalid_char:
e_invalid_char_in_number:
	error_info->line = lineno;
	error_info->column = (p - first_column) / sizeof(char) + 1;
	free(self->tokens);
	free(self->_mem);
e_oom_strdup:
	free(self);
e_oom_toquence:
	return NULL;
}


static void
toquence_del(toquence* self)
{
	free(self->_mem);
	free(self->tokens);
	free(self);
}




/* Misc functions for syntax analysis */


/* Return true for no more tokens */
static int
toquence_end(const toquence* self)
{
	return !(self->current < self->num);
}

/* Skip to next token */
static void
toquence_next(toquence* self)
{
	self->current++;
}

/* Return the current token */
static const token*
toquence_current(const toquence* self)
{
	return toquence_end(self) ? NULL : (self->tokens + self->current);
}

static token_id
toquence_current_id(const toquence* self)
{
	return toquence_end(self)
	       ? TI_END
	       : (self->tokens + self->current)->id;
}

static const char*
toquence_current_attr(const toquence* self)
{
	return toquence_end(self)
	       ? NULL
	       : (self->tokens + self->current)->attr;
}

#if 0  /* not used */
/* Return the last token  */
static const token*
toquence_last(const toquence* self)
{
	return (0 <= self->num - 1) ? (self->tokens + self->num - 1) : NULL;
}
#endif

/* Return true for the id of the current token matching a given id */
static int
toquence_match(toquence* self, token_id id)
{
	return (!toquence_end(self)) && (self->tokens[self->current].id == id);
}

/* Abbreviation */
static int
toquence_matchnext(toquence* self, token_id id)
{
	if (toquence_match(self, id)) {
		toquence_next(self);
		return !0;
	} else {
		return 0;
	}
}




/**
 * Functions corresponding to each production.
 *
 * Return a new command for success or NULL for failure,
 * and store the error information to attended_token and error_message.
 */

		/* forward */
#define first_simple_command(id) ((id) == TI_NAME)
#define first_multi_command(id) ((id) == TI_LBRA)
#define first_loop_command(id) ((id) == TI_LOOP)
#define first_random_command(id) ((id) == TI_RANDOM)
#define first_thread_command(id) ((id) == TI_THREAD)
#define first_command(id) (    first_simple_command(id) \
                            || first_multi_command(id) \
                            || first_loop_command(id) \
                            || first_random_command(id) \
                            || first_thread_command(id) \
                          )
static BBCmdCommand* prod_command( toquence*, const BBCmdCompiler*,
                                   const token**, const char** );


static BBCmdCommand*
prod_simple_command( toquence* toq,
                     const BBCmdCompiler* compiler,
                     const token** attended_token,
                     const char** error_message )
{
	const BBCmdCommandDescriptor* descriptor;
	BBCmdCommand* command;
	int ac;
	const char** av;
	const char* e_i;
	const token* name_token;


	command = NULL;
	av = NULL;

	if (!toquence_match(toq, TI_NAME)) {
		*attended_token = toquence_current(toq);
		*error_message = "Unexpected token";
		goto ERR1;
	}

	ac = 1;
	av = (const char**)malloc(sizeof(const char*) * 1);
	if (av == NULL) {
		*attended_token = toquence_current(toq);
		*error_message = "Out of memory";
		goto ERR2;
	}
	name_token = toquence_current(toq);
	av[0] = name_token->attr;
	toquence_next(toq);

	if (toquence_match(toq, TI_LPAR)) {
		const token* lpar_token;

		lpar_token = toquence_current(toq);
		do {
			const char** av_t;

			toquence_next(toq);
			if (!( toquence_match(toq, TI_STRING)
			       || toquence_match(toq, TI_NUMBER) ))
				break;

			av_t = (const char**)realloc
			           (av, sizeof(const char*) * (ac+1));
			if (av_t == NULL) {
				*attended_token = toquence_current(toq);
				*error_message = "Out of memory";
				goto ERR3;
			}
			av = av_t;
			av[ac] = toquence_current_attr(toq);
			ac++;

			toquence_next(toq);
		} while (toquence_match(toq, TI_COMMA));

		if (!toquence_match(toq, TI_RPAR)) {
			*attended_token = lpar_token;
			*error_message = toquence_end(toq)
			                 ? "Missing right paren"
			                 : "Unexpected token";
			goto ERR4;
		}
		toquence_next(toq);
	}

	descriptor = _BBCmdCompiler_FindDescriptor(compiler, av[0]);
	if (descriptor == NULL) {
		*attended_token = name_token;
		*error_message = "Unknown command";
		goto ERR5;
	}

	command = (BBCmdCommand*)malloc(descriptor->cmd_size);
	if (command == NULL) {
		*attended_token = name_token;
		*error_message = "Out of memory";
		goto ERR6;
	}
	BBCmdCommand_INIT(command, descriptor);

	/* av[0] = descriptor->cmd_name;  # replace alias */
	e_i = (*(descriptor->cmd_initialize))(command, ac, av, compiler);
	if (e_i != NULL) {
		*attended_token = name_token;
		*error_message = e_i;
		goto ERR7;
	}

	goto SUCCESS;
ERR7:
	BBCmdCommand_Del(command);
	command = NULL;
ERR6:
ERR5:
ERR4:
ERR3:
ERR2:
ERR1:
SUCCESS:
	free(av);
	return command;
}


static BBCmdCommand*
prod_multi_command( toquence* toq,
                    const BBCmdCompiler* compiler,
                    const token** attended_token,
                    const char** error_message )
{
	BBCmdCommand* multicommand;
	BBCmdCommand* command;

	multicommand = BBCmdMultiCommand_New();
	if (multicommand == NULL) {
		*attended_token = toquence_current(toq);
		*error_message = "Out of memory";
		goto ERR1;
	}

	if (!toquence_matchnext(toq, TI_LBRA)) {
		*attended_token = toquence_current(toq);
		*error_message = "Unexpected token";
		goto ERR2;
	}

	while (first_command(toquence_current_id(toq))) {
		command = prod_command
			      (toq, compiler, attended_token, error_message);
		if (command == NULL)
			goto ERR3;
		if (!BBCmdMultiCommand_Add(multicommand, command)) {
			BBCmdCommand_Del(command);
			*attended_token = toquence_current(toq);
			*error_message = "Out of memory";
			goto ERR4;
		}
	}

	if (!toquence_matchnext(toq, TI_RBRA)) {
		*attended_token = toquence_current(toq);
		*error_message = "Missing right brace";
		goto ERR5;
	}

	return multicommand;

ERR5:
ERR4:
ERR3:
ERR2:
	BBCmdCommand_Del(multicommand);
ERR1:
	return NULL;
}


static BBCmdCommand*
prod_loop_command( toquence* toq,
                   const BBCmdCompiler* compiler,
                   const token** attended_token,
                   const char** error_message )
{
	BBCmdCommand* loopcommand;
	BBCmdCommand* command;
	const token* loop_token;
	int num;

	loop_token = toquence_current(toq);
	if (!toquence_matchnext(toq, TI_LOOP)) {
		*attended_token = toquence_current(toq);
		*error_message = "Unexpected token";
		goto ERR1;
	}

	if (!toquence_match(toq, TI_NUMBER)) {
		*attended_token = toquence_current(toq);
		*error_message = "Unexpected token";
		goto ERR2;
	}
	num = number_attr_as_int(toquence_current_attr(toq));
	if (num < 0) {
		*attended_token = toquence_current(toq);
		*error_message = "Loop number must be zero or positive";
		goto ERR3;
	}
	toquence_next(toq);

	if (!first_command(toquence_current_id(toq))) {
		*attended_token = toquence_current(toq);
		*error_message = "Unexpected token";
		goto ERR4;
	}
	command = prod_command
		      (toq, compiler, attended_token, error_message);
	if (command == NULL)
		goto ERR5;

	loopcommand = BBCmdLoopCommand_FromIntAndCommand(num, command);
	if (loopcommand == NULL) {
		*attended_token = loop_token;
		*error_message = "Out of memory";
		goto ERR6;
	}

	return loopcommand;

ERR6:
	BBCmdCommand_Del(command);
ERR5:
ERR4:
ERR3:
ERR2:
ERR1:
	return NULL;
}


static BBCmdCommand*
prod_random_command( toquence* toq,
                     const BBCmdCompiler* compiler,
                     const token** attended_token,
                     const char** error_message )
{
	BBCmdCommand* random_command;
	BBCmdCommand* command;
	const token* random_token;

	random_token = toquence_current(toq);
	if (!toquence_matchnext(toq, TI_RANDOM)) {
		*attended_token = toquence_current(toq);
		*error_message = "Unexpected token";
		goto ERR1;
	}

	if (!first_multi_command(toquence_current_id(toq))) {
		*attended_token = toquence_current(toq);
		*error_message = "Unexpected token";
		goto ERR2;
	}
	command = prod_multi_command
		      (toq, compiler, attended_token, error_message);
	if (command == NULL)
		goto ERR3;

	random_command = BBCmdRandomCommand_New(command);
	if (random_command == NULL) {
		*attended_token = random_token;
		*error_message = "Out of memory";
		goto ERR4;
	}

	return random_command;

ERR4:
	BBCmdCommand_Del(command);
ERR3:
ERR2:
ERR1:
	return NULL;
}


static BBCmdCommand*
prod_thread_command( toquence* toq,
                     const BBCmdCompiler* compiler,
                     const token** attended_token,
                     const char** error_message )
{
	BBCmdCommand* thread_command;
	BBCmdCommand* command;
	const token* thread_token;

	thread_token = toquence_current(toq);
	if (!toquence_matchnext(toq, TI_THREAD)) {
		*attended_token = toquence_current(toq);
		*error_message = "Unexpected token";
		goto ERR1;
	}

	if (!first_command(toquence_current_id(toq))) {
		*attended_token = toquence_current(toq);
		*error_message = "Unexpected token";
		goto ERR2;
	}
	command = prod_command(toq, compiler, attended_token, error_message);
	if (command == NULL)
		goto ERR3;

	thread_command = BBCmdThreadCommand_New(command);
	if (thread_command == NULL) {
		*attended_token = thread_token;
		*error_message = "Out of memory";
		goto ERR4;
	}

	return thread_command;

ERR4:
	BBCmdCommand_Del(command);
ERR3:
ERR2:
ERR1:
	return NULL;
}


static BBCmdCommand*
prod_command( toquence* toq,
              const BBCmdCompiler* compiler,
              const token** attended_token,
              const char** error_message )
{
  if (first_simple_command(toquence_current_id(toq)))
    return prod_simple_command(toq, compiler, attended_token, error_message);
  else if (first_multi_command(toquence_current_id(toq)))
    return prod_multi_command(toq, compiler, attended_token, error_message);
  else if (first_loop_command(toquence_current_id(toq)))
    return prod_loop_command(toq, compiler, attended_token, error_message);
  else if (first_random_command(toquence_current_id(toq)))
    return prod_random_command(toq, compiler, attended_token, error_message);
  else if (first_thread_command(toquence_current_id(toq)))
    return prod_thread_command(toq, compiler, attended_token, error_message);
  else {
    *attended_token = toquence_current(toq);
    *error_message = "Unexpected token";
    return NULL;
  }
}





BBCmdAPI_FUNC(BBCmdCommand*)
BBCmdCompiler_CompileFromString( const BBCmdCompiler* self,
                                 const char* script,
                                 int* error_line,
                                 int* error_column,
                                 const char** error_message )
{
	BBCmdCommand* multicommand;
	toquence* toq;
	toquence_error error_info;
	const token* attended_token;

	toq = toquence_from_string(script, &error_info);
	if (toq == NULL) {
		*error_line = error_info.line;
		*error_column = error_info.column;
		*error_message = error_info.message;
		return NULL;
	}

	multicommand = BBCmdMultiCommand_New();
	if (multicommand == NULL) {
		attended_token = NULL;
		*error_message = "Out of memory";
		goto ERR;
	}

	while (!toquence_end(toq)) {
		BBCmdCommand* command;

		command = prod_command
			  (toq, self, &attended_token, error_message);
		if (command == NULL) goto ERR;

		if (!BBCmdMultiCommand_Add(multicommand, command)) {
			BBCmdCommand_Del(command);
			*error_message = "Out of memory";
			goto ERR;
		}
	}

	toquence_del(toq);

	return multicommand;

ERR:
	*error_line = (attended_token == NULL ? 0 : attended_token->line);
	*error_column = (attended_token == NULL ? 0 : attended_token->column);
	BBCmdCommand_Del(multicommand);
	toquence_del(toq);
	return NULL;
}




#if 0
/* Old compiler code */

BBCmdAPI_FUNC(BBCmdCommand*)
BBCmdCompiler_CompileFromString(const BBCmdCompiler* self, const char* script)
{
	BBCmdCommand* multicommand;
	toquence* toq;
	toquence_error error_info;
	const char* error_message;

	const char** av;
	const token* attended_token;

	toq = toquence_from_string(script, &error_info);
	if (toq == NULL) {
		char buf[80];
		snprintf( buf, BBCmd_arraysizeof(buf),
		          "<const char* %p>:%d:%d: %s",
		          script, error_info.line, error_info.column,
		          error_info.message );
		buf[BBCmd_arraysizeof(buf)-1] = '\0';
		return BBCmdErrorCommand_FromString(buf);
	}

	multicommand = BBCmdMultiCommand_New();
	if (!BBCmdCommand_IsValid(multicommand)) {
		error_message = BBCmdCommand_StrError(multicommand);
		attended_token = NULL;
		goto ERR1;
	}

	while (!toquence_end(toq)) {
		const BBCmdCommandDescriptor* descriptor;
		BBCmdCommand* command;
		int ac;
		int e_a;
		const char* e_i;

		attended_token = toquence_current(toq);
		if (!toquence_match(toq, TI_NAME)) {
			error_message = "Unexpected token (name is expected)";
			goto ERR1;
		}

		ac = 1;
		av = (const char**)malloc(sizeof(const char*) * 1);
		if (av == NULL) {
			error_message = "Out of memory when parsing (#1)";
			goto ERR2;
		}

		av[0] = attended_token->attr;
		toquence_next(toq);

		if (toquence_match(toq, TI_LPAR)) {
			do {
				const char** av_t;

				toquence_next(toq);
				if (!( toquence_match(toq, TI_STRING)
				       || toquence_match(toq, TI_NUMBER) ))
					break;

				av_t = (const char**)realloc
				(av, sizeof(const char*) * (ac+1));
				if (av_t == NULL) {
					error_message
					  = "Out of memory when parsing (#2)";
					goto ERR3;
				}
				av = av_t;
				av[ac] = toquence_current(toq)->attr;
				ac++;

				toquence_next(toq);
			} while (toquence_match(toq, TI_COMMA));

			if (!toquence_match(toq, TI_RPAR)) {
				error_message = toquence_end(toq)
				                ? "Missing right paren"
				                : "Unexpected token "
				                  "(right paren is expected)";
				goto ERR3;
			}
			toquence_next(toq);
		}

		descriptor = _BBCmdCompiler_FindDescriptor(self, av[0]);
		if (descriptor == NULL) {
			error_message = "Unknown command";
			goto ERR2;
		}

		command = (BBCmdCommand*)malloc(descriptor->cmd_size);
		if (command == NULL) {
			error_message = "Out of memory when compiling";
			goto ERR2;
		}
		BBCmdCommand_INIT(command, descriptor);

		/* av[0] = descriptor->cmd_name;  # replace alias */
		e_i = (*(descriptor->cmd_initialize))(command, ac, av, self);
		if (e_i != NULL) {
			BBCmdCommand_Del(command);
			error_message = e_i;
			goto ERR2;
		}
		free(av);

		e_a = BBCmdMultiCommand_Add(multicommand, command);
		if (!e_a) {
			BBCmdCommand_Del(command);
			error_message
			    = "Out of memory - too many commands to compile";
			goto ERR2;
		}
	}

	toquence_del(toq);

	return multicommand;

ERR3:
	attended_token = toquence_end(toq) ? toquence_last(toq)
	                                        : toquence_current(toq);
ERR2:
	free(av);
ERR1:
	{
	  char buf[80];

	  snprintf( buf, BBCmd_arraysizeof(buf),
	            "<const char* %p>:%d:%d: %s",
	            script,
	            (attended_token == NULL ? 0 : attended_token->line),
	            (attended_token == NULL ? 0 : attended_token->column),
	            error_message );
	  buf[BBCmd_arraysizeof(buf)] = '\0';
	  error_message = buf;

	  BBCmdCommand_Del(multicommand);
	  toquence_del(toq);
	  return BBCmdErrorCommand_FromString(error_message);
	}
}
#endif




/* __END__ */
