/* Implementation of `file' command */

#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "File_p.h"




#define ABS(n) ((n) < 0 ? -(n) : (n))


static int
counts_char(const char* s, char c)
{
	int n;

	n = 0;
	while (*s != '\0') {
		if (*s == c)
			n++;
		s++;
	}

	return n;
}




static const char*
_BBCmdFileCommand_initialize( BBCmdCommand* _self,
                              int ac, const char* const* av,
                              const BBCmdCompiler* compiler )
{
	BBCmdFileCommand* self;
	FILE* file;
	long size;
	char* script;
	const char* ret = NULL;
	int r;

	assert(_self->cmd_descriptor == BBCmdFileCommandDescriptor);
	self = (BBCmdFileCommand*)_self;

	self->fi_command = NULL;

	if (ac - 1 != 1) {ret = "Incorrect number of arguments";  goto ERR1;}

	/* get file size */
	file = fopen(av[1], "rb");
	if (file == NULL)
		{ret = "Unable to open the specified file (1)";  goto ERR1;}

	r = fseek(file, 0L, SEEK_END);
	if (r != 0) {ret = "Unable to fseek the specified file";  goto ERR2;}

	size = ftell(file);
	if (size == -1)
		{ret = "Unable to ftell the specified file";  goto ERR2;}

	fclose(file);

	/* read file */
	file = fopen(av[1], "rt");
	if (file == NULL)
		{ret = "Unable to open the specified file (2)";  goto ERR1;}

	script = (char*)malloc(sizeof(char) * (size+1));
	if (script == NULL) {ret = "Out of memory";  goto ERR2;}

	r = fread(script, sizeof(char), size, file);
	script[r] = '\0';
	if ( ferror(file)
	     || ((r != size) && (ABS(size - r) != counts_char(script, '\n'))) )
	{
		ret = "Failed to read the specified file";
		goto ERR3;
	}

	if (!strcmp(av[0], "broamrelay")) {
		int escaped_char_num = 0;
		int line_num = 0;
		char* p;
		char* t;
		char* i;
		char* o;

		for (p = script; *p != '\0'; p++) {
			if (*p == '\\' || *p == '"')
				escaped_char_num++;
			else if (*p == '\n')
				line_num++;
			else
				; /* nop */
		}

		if (1 <= size && script[size-1] != '\n')
			line_num++;

		t = malloc( sizeof(char) * ( size + escaped_char_num
		                             + line_num * (5 + 2 + 2) + 1 ) );
		if (t == NULL) {ret = "Out of memory"; goto ERR3;}

		if (1 <= line_num) strcpy(t, "broam(\"");
		else               t[0] = '\0';

		o = t + strlen(t);
		i = script;
		while (*i != '\0') {
			switch (*i) {
			default:
				*o = *i;
				o++;
				break;
			case '\\': case '"':
				*o = '\\';
				o++;
				*o = *i;
				o++;
				break;
			case '\n':
				strcpy(o, "\")\n");
				o += 3;
				if (*(i + 1) != '\0') {
					strcpy(o, "broam(\"");
					o += 7;
				}
				break;
			}
			i++;
		}

		*o = '\0';
		if (1 <= line_num && script[size-1] != '\n')
			strcpy(o, "\")");

		free(script);
		script = t;
	}

	{
	  int line;
	  int column;
	  const char* message;

	  self->fi_command = BBCmdCompiler_CompileFromString
	                         (compiler, script, &line, &column, &message);
	  if (self->fi_command == NULL)
	  	{ret = "Failed to compile the specified file";  goto ERR3;}
	}

ERR3:
	free(script);
ERR2:
	fclose(file);
ERR1:
	return ret;
}


static void
_BBCmdFileCommand_del(BBCmdCommand* _self)
{
	BBCmdFileCommand* self;

	assert(_self->cmd_descriptor == BBCmdFileCommandDescriptor);
	self = (BBCmdFileCommand*)_self;

	BBCmdCommand_Del(self->fi_command);
}


static int
_BBCmdFileCommand_execute(const BBCmdCommand* _self)
{
	BBCmdFileCommand* self;

	assert(_self->cmd_descriptor == BBCmdFileCommandDescriptor);
	self = (BBCmdFileCommand*)_self;

	return BBCmdCommand_Execute(self->fi_command);
}




static const char* _BBCmdFileCommand_alias[] = {
	"broamrelay",
	NULL
};

const BBCmdCommandDescriptor _BBCmdFileCommandDescriptor = {
	"file",
	_BBCmdFileCommand_alias,
	"(script) --> result of script\n"
		"Read the file named `script' and execute it.\n"
		"Don't write endless recursive loop.\n"
		"For example, writing `file(\"samefile\")' in `samefile',\n"
		"this will be failed.\n"
		"`broamrelay(text)' will send bro@m per line in `text'.",
	sizeof(BBCmdFileCommand),
	_BBCmdFileCommand_initialize,
	_BBCmdFileCommand_del,
	_BBCmdFileCommand_execute
};




/* __END__ */
