/* Implementation of `loop' command */

#include <assert.h>
#include <stdlib.h>

#include "Loop_p.h"




static const char*
_BBCmdLoopCommand_initialize( BBCmdCommand* self,
                              int ac, const char* const* av,
                              const BBCmdCompiler* compiler )
{
	/* deny to make a new instance from a compiler */
	abort();
	return "BUG (_BBCmdLoopCommand_initialize)";
}


static void
_BBCmdLoopCommand_del(BBCmdCommand* _self)
{
	BBCmdLoopCommand* self;

	assert(_self->cmd_descriptor == BBCmdLoopCommandDescriptor);
	self = (BBCmdLoopCommand*)_self;

	BBCmdCommand_Del(self->lo_command);
}


static int
_BBCmdLoopCommand_execute(const BBCmdCommand* _self)
{
	BBCmdLoopCommand* self;
	int i;
	int rc = BBCmd_RC_TRUE;

	assert(_self->cmd_descriptor == BBCmdLoopCommandDescriptor);
	self = (BBCmdLoopCommand*)_self;

	for (i = 0; i < self->lo_num; i++) {
		rc = _BBCmdCommand_Execute(self->lo_command);

		if (rc == BBCmd_RC_BREAK)
			break;
		if (rc == BBCmd_RC_CONTINUE)
			continue;
	}

	if (rc == BBCmd_RC_BREAK || rc == BBCmd_RC_CONTINUE)
		rc = BBCmd_RC_TRUE;

	return rc;
}




const BBCmdCommandDescriptor _BBCmdLoopCommandDescriptor = {
	"Loop",
	NULL,
	"Built-in command to execute a command many times.\n"
	"This will return the result of the last command.",
	sizeof(BBCmdLoopCommand),
	_BBCmdLoopCommand_initialize,
	_BBCmdLoopCommand_del,
	_BBCmdLoopCommand_execute
};




BBCmdCommand*
BBCmdLoopCommand_FromIntAndCommand(int num, BBCmdCommand* command)
{
	BBCmdLoopCommand* self;

	if (num < 0) return NULL;

	self = (BBCmdLoopCommand*)malloc(sizeof(*self));
	if (self == NULL) return NULL;

	BBCmdCommand_INIT(self, BBCmdLoopCommandDescriptor);

	self->lo_num = num;
	self->lo_command = command;

	return (BBCmdCommand*)self;
}




/* __END__ */
