/* Implementation of {...} */

#include <assert.h>
#include <stdlib.h>

#include "Multi_p.h"




static const char*
_BBCmdMultiCommand_initialize( BBCmdCommand* self,
                               int ac, const char* const* av,
                               const BBCmdCompiler* compiler )
{
	/* deny to make a new instance from a compiler */
	abort();
	return "BUG (_BBCmdMultiCommand_initialize)";
}


static void
_BBCmdMultiCommand_del(BBCmdCommand* _self)
{
	BBCmdMultiCommand* self;
	int i;

	assert(_self->cmd_descriptor == BBCmdMultiCommandDescriptor);
	self = (BBCmdMultiCommand*)_self;

	for (i = 0; i < self->mu_list_size; i++)
		BBCmdCommand_Del(self->mu_list[i]);

	free(self->mu_list);
}


static int
_BBCmdMultiCommand_execute(const BBCmdCommand* _self)
{
	BBCmdMultiCommand* self;
	int i;
	int rc = BBCmd_RC_TRUE;

	assert(_self->cmd_descriptor == BBCmdMultiCommandDescriptor);
	self = (BBCmdMultiCommand*)_self;

	for (i = 0; i < self->mu_list_size; i++) {
		rc = _BBCmdCommand_Execute(self->mu_list[i]);

		if (rc == BBCmd_RC_BREAK || rc == BBCmd_RC_CONTINUE)
			break;
	}

	return rc;
}




const BBCmdCommandDescriptor _BBCmdMultiCommandDescriptor = {
	"Multi",
	NULL,
	"Built-in command to regard many commands as a command.\n"
	"This will return the result of the last command.",
	sizeof(BBCmdMultiCommand),
	_BBCmdMultiCommand_initialize,
	_BBCmdMultiCommand_del,
	_BBCmdMultiCommand_execute
};




BBCmdCommand*
BBCmdMultiCommand_New(void)
{
	BBCmdMultiCommand* self;

	self = (BBCmdMultiCommand*)malloc(sizeof(*self));
	if (self == NULL) return NULL;

	BBCmdCommand_INIT(self, BBCmdMultiCommandDescriptor);

	self->mu_list = NULL;
	self->mu_list_size = 0;

	return (BBCmdCommand*)self;
}


int
BBCmdMultiCommand_Add(BBCmdCommand* _self, BBCmdCommand* command)
{
	BBCmdMultiCommand* self;
	BBCmdCommand** new_list;

	assert(_self->cmd_descriptor == BBCmdMultiCommandDescriptor);
	self = (BBCmdMultiCommand*)_self;

	new_list = (BBCmdCommand**)realloc(
		self->mu_list,
	        sizeof(self->mu_list[0]) * (self->mu_list_size+1)
	);
	if (new_list == NULL) return 0;

	self->mu_list = new_list;
	self->mu_list[self->mu_list_size] = command;
	self->mu_list_size++;

	return !0;
}




/* __END__ */
