/* Implementation of `pause' command */

#include <assert.h>
#include <stdlib.h>
#include <string.h>

#define WIN32_LEAN_AND_MEAN
#include <windows.h>

#include "Pause_p.h"




static const char*
_BBCmdPauseCommand_initialize( BBCmdCommand* _self,
                               int ac, const char* const* av,
                               const BBCmdCompiler* compiler )
{
	BBCmdPauseCommand* self;

	assert(_self->cmd_descriptor == BBCmdPauseCommandDescriptor);
	self = (BBCmdPauseCommand*)_self;

	if (ac - 1 < 0) return "Too few arguments";

	if (!strcmp(av[0], "pause")) {
		if (1 < ac - 1) return "Too many arguments";
		self->pa_seconds = ((ac - 1 == 0) ? 1 : atoi(av[1]));
		if (self->pa_seconds < 0)
			return "`sec' must be zero or positive";
	} else if (!strcmp(av[0], "softpause")) {
		if (1 < ac - 1) return "Too many arguments";
		self->pa_seconds = ((ac - 1 == 0) ? 1 : atoi(av[1]));
		if (self->pa_seconds <= 0)
			return "`sec' must be positive";
	} else if (!strcmp(av[0], "hardpause")) {
		if (0 < ac - 1) return "Too many arguments";
		self->pa_seconds = 0;
	} else {
		return "Unknown command (maybe BBCmd's bug)";
	}

	return NULL;
}


static void
_BBCmdPauseCommand_del(BBCmdCommand* self)
{
	/* nothing to do */
}


static int
_BBCmdPauseCommand_execute(const BBCmdCommand* _self)
{
	BBCmdPauseCommand* self;

	assert(_self->cmd_descriptor == BBCmdPauseCommandDescriptor);
	self = (BBCmdPauseCommand*)_self;

	if (self->pa_seconds == 0) {
		MessageBox( NULL, "Press OK to continue.", "BBCmd",
			    MB_OK | MB_ICONINFORMATION | MB_TASKMODAL );
	} else {
		Sleep(self->pa_seconds * 1000);
	}

	return BBCmd_RC_TRUE;
}




static const char* _BBCmdPauseCommand_alias[] = {
	"softpause",
	"hardpause",
	NULL
};

const BBCmdCommandDescriptor _BBCmdPauseCommandDescriptor = {
	"pause",
	_BBCmdPauseCommand_alias,
	"([sec=1]) --> true\n"
	    "Sleep `sec' seconds.\n"
	    "If `sec' is 0, then show a dialog\n"
	    "and sleep until the dialog is dismissed.\n"
	    "`softpause' is like `pause(sec)' but `sec' must be positive.\n"
	    "`hardpause' is same as `pause(0)'.",
	sizeof(BBCmdPauseCommand),
	_BBCmdPauseCommand_initialize,
	_BBCmdPauseCommand_del,
	_BBCmdPauseCommand_execute
};




/* __END__ */
