/* Implementation of `sound' command */

#include <assert.h>
#include <stdlib.h>
#include <string.h>

#include "Sound_p.h"




static const char*
_BBCmdSoundCommand_initialize( BBCmdCommand* _self,
                               int ac, const char* const* av,
                               const BBCmdCompiler* compiler )
{
	BBCmdSoundCommand* self;

	assert(_self->cmd_descriptor == BBCmdSoundCommandDescriptor);
	self = (BBCmdSoundCommand*)_self;

	self->so_wavefile = NULL;

	if (ac-1 < 1 || 2 < ac-1) return "Incorrect number of arguments";

	self->so_wavefile = strdup(av[1]);
	if (self->so_wavefile == NULL)
		return "Out of memory (_BBCmdSoundCommand_initialize)";

	if (!strcmp(av[0], "syncsound")) {
		self->so_flags = SND_FILENAME | SND_SYNC;
		if (ac - 1 != 1) return "Incorrect number of arguments";
	} else if (!strcmp(av[0], "asyncsound")) {
		self->so_flags = SND_FILENAME | SND_ASYNC;
		if (ac - 1 != 1) return "Incorrect number of arguments";
	} else {
		self->so_flags = SND_FILENAME;
		if (ac - 1 == 2) {
			if (!strcmp(av[2], "sync"))
				self->so_flags |= SND_SYNC;
			else if (!strcmp(av[2], "async"))
				self->so_flags |= SND_ASYNC;
			else
				return "Invalid option";
		} else {
			self->so_flags |= SND_SYNC;
		}
	}

	return NULL;
}


static void
_BBCmdSoundCommand_del(BBCmdCommand* _self)
{
	BBCmdSoundCommand* self;

	assert(_self->cmd_descriptor == BBCmdSoundCommandDescriptor);
	self = (BBCmdSoundCommand*)_self;

	free(self->so_wavefile);
}


static int
_BBCmdSoundCommand_execute(const BBCmdCommand* _self)
{
	BBCmdSoundCommand* self;

	assert(_self->cmd_descriptor == BBCmdSoundCommandDescriptor);
	self = (BBCmdSoundCommand*)_self;

	return (PlaySound(self->so_wavefile, NULL, self->so_flags) == TRUE)
	       ? BBCmd_RC_TRUE : BBCmd_RC_FALSE;
}




static const char* _BBCmdSoundCommand_alias[] = {
	"syncsound",
	"asyncsound",
	NULL
};

const BBCmdCommandDescriptor _BBCmdSoundCommandDescriptor = {
	"sound",
	_BBCmdSoundCommand_alias,
	"(wavafile [, option=\"sync\"])\n"
		"Play `wavefile' synchronously or asynchronously.\n"
		"`syncsound'  is short for `sound(wavefile, \"sync\" )'.\n"
		"`asyncsound' is short for `sound(wavefile, \"async\")'.",
	sizeof(BBCmdSoundCommand),
	_BBCmdSoundCommand_initialize,
	_BBCmdSoundCommand_del,
	_BBCmdSoundCommand_execute
};




/* __END__ */
