/* Implementation of `workspace' command */

#include <assert.h>
#include <stdlib.h>

#include <BBApi-C.h>

#include "Workspace_p.h"




static const char*
_BBCmdWorkspaceCommand_initialize( BBCmdCommand* _self,
                              int ac, const char* const* av,
                              const BBCmdCompiler* compiler )
{
	BBCmdWorkspaceCommand* self;

	assert(_self->cmd_descriptor == BBCmdWorkspaceCommandDescriptor);
	self = (BBCmdWorkspaceCommand*)_self;

	if (ac - 1 != 1 && ac - 1 != 3) return "Incorrect number of arguments";

	if (ac - 1 == 1) {
		self->wo_destination = atoi(av[1]);
		if (self->wo_destination < 1)
			return "1st value must be positive";
		self->wo_destination--;

		self->wo_rows = 0;
		self->wo_columns = 0;
	} else {
		self->wo_destination = atoi(av[1]);
		if (self->wo_destination < 1 || 9 < self->wo_destination)
			return "1st value must be 1, 2, ... or 9.";

		self->wo_rows = atoi(av[2]);
		if (self->wo_rows < 1) return "2nd value must be positive";

		self->wo_columns = atoi(av[3]);
		if (self->wo_columns < 1) return "3rd value must be positive";
	}

	return NULL;
}


static void
_BBCmdWorkspaceCommand_del(BBCmdCommand* _self)
{
	/* nothing to do */
}


static int m(int l, int r)
{
	while (l < 0) l += r;
	return l % r;
}
static int upof   (int n, int r, int c) {return m(n-c, r*c);}
static int downof (int n, int r, int c) {return m(n+c, r*c);}
static int leftof (int n, int r, int c) {return m(n%c-1, c) + n/c*c;}
static int rightof(int n, int r, int c) {return m(n%c+1, c) + n/c*c;}

static int
_BBCmdWorkspaceCommand_execute(const BBCmdCommand* _self)
{
	BBCmdWorkspaceCommand* self;
	HWND core;
	int n;

	assert(_self->cmd_descriptor == BBCmdWorkspaceCommandDescriptor);
	self = (BBCmdWorkspaceCommand*)_self;

	core = BBCmd_FindCore();
	if (core == NULL) return BBCmd_RC_FALSE;

	if (self->wo_rows != 0 && self->wo_columns != 0) {
		n = BBCmd_GetCurrentWorkspaceNumber();
		if (n < 0) return BBCmd_RC_FALSE;

		switch (self->wo_destination) {
		default: break;

		case 1:
			n = downof (n, self->wo_rows, self->wo_columns);
			n = leftof (n, self->wo_rows, self->wo_columns);
			break;
		case 2:
			n = downof (n, self->wo_rows, self->wo_columns);
			break;
		case 3:
			n = downof (n, self->wo_rows, self->wo_columns);
			n = rightof(n, self->wo_rows, self->wo_columns);
			break;

		case 4:
			n = leftof (n, self->wo_rows, self->wo_columns);
			break;
		case 5:
			break;
		case 6:
			n = rightof(n, self->wo_rows, self->wo_columns);
			break;

		case 7:
			n = upof   (n, self->wo_rows, self->wo_columns);
			n = leftof (n, self->wo_rows, self->wo_columns);
			break;
		case 8:
			n = upof   (n, self->wo_rows, self->wo_columns);
			break;
		case 9:
			n = upof   (n, self->wo_rows, self->wo_columns);
			n = rightof(n, self->wo_rows, self->wo_columns);
			break;
		}
	} else {
		n = self->wo_destination;
	}

	PostMessage(core, BB_WORKSPACE, 4, n);

	return BBCmd_RC_TRUE;
}




const BBCmdCommandDescriptor _BBCmdWorkspaceCommandDescriptor = {
	"workspace",
	NULL,
	"(n [, rows, columns])\n"
		"Move the `n'th workspace.\n"
		"Workspaces are enumrated like 1, 2, 3... n.\n"
		"See README for `rows' and `columns'.",
	sizeof(BBCmdWorkspaceCommand),
	_BBCmdWorkspaceCommand_initialize,
	_BBCmdWorkspaceCommand_del,
	_BBCmdWorkspaceCommand_execute
};




/* __END__ */
